/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * Separation.cpp
 *
 *  Created on: 2010/11/11
 *      Author: HARK Development Team
 */

#include <iostream>

#include <Separation.hpp>
#include <InterauralDifference.hpp>
#include <DirectionPassFilter.hpp>
#include <DimensionPromise/dimpro.hpp>
#include <AngleConstants.hpp>

using namespace std;
using namespace NSDimensionPromise;

namespace NSHarkBinaural
{

EpipolarSeparation::LRSpectra::ptr_t EpipolarSeparation::separate(NSDimensionPromise::Angle dir, NSDimensionPromise::Angle range)
{
    IPD::ptr_t ipd(new IPD(this->GetInputSpectra(), Channel::R)); // 両耳間位相差を求める
    IID::ptr_t iid(new IID(this->GetInputSpectra(), Channel::R)); // 両耳間強度差を求める

    // ダミーの入力信号パワースペクトル
    Unspectrum::ptr_t signalPowerSpectrum(new Unspectrum(this->GetInputSpectra()->Left()->Size()));
    // ダミーのルームノイズパワースペクトル
    Unspectrum::ptr_t roomnoisePowerSpectrum(new Unspectrum(this->GetInputSpectra()->Left()->Size()));

    PassBand::ptr_t band(new PassBand(ipd->Size()));
    SetPassBandFlags(band, PassBandFlag::CutOff);
    band->SetPassBandEpipolar(
        PassBandSetMode::PassOnly,
        this->GetSeparationSetting(),
        dir, range, ipd,
        signalPowerSpectrum, roomnoisePowerSpectrum,
        Length::FromMillimeter(1000), Length::FromMillimeter(90));

    // スペクトル抽出機
    BandExtractor extractor(this->GetInputSpectra());

    LRSpectra::ptr_t separatedSpectra = extractor.Extract(band);
    return separatedSpectra;
}

HRTFSeparation::LRSpectra::ptr_t HRTFSeparation::separate(NSDimensionPromise::Angle dir, NSDimensionPromise::Angle range)
{
    IPD::ptr_t ipd(new IPD(this->GetInputSpectra(), Channel::R)); // 両耳間位相差を求める
    IID::ptr_t iid(new IID(this->GetInputSpectra(), Channel::R)); // 両耳間強度差を求める

    // 信号のパワースペクトル
    size_t len = this->GetInputSpectra()->Left()->Size();
    LRUnspectra::ptr_t lrpower = LRSignalFactory<Unspectrum::ElementType>::Create(len);
    Spectrum& sigL = *this->GetInputSpectra()->Left();
    Spectrum& sigR = *this->GetInputSpectra()->Right();
    Unspectrum& powL = *lrpower->Left();
    Unspectrum& powR = *lrpower->Right();
    for (size_t i = 0; i < len; ++i)
    {
        powL[i] = Decibel(sigL[i]);
        powR[i] = Decibel(sigR[i]);
    }

    PassBand::ptr_t band(new PassBand(ipd->Size()));
    SetPassBandFlags(band, PassBandFlag::CutOff);
    band->SetPassBandHRTF(
        PassBandSetMode::PassOnly,
        this->GetSeparationSetting(),
        dir, range, ipd, iid,
        lrpower,
        bgnoise,
        iipds,
        thresDB);

    // スペクトル抽出機
    BandExtractor extractor(this->GetInputSpectra());

    LRSpectra::ptr_t separatedSpectra = extractor.Extract(band);
    return separatedSpectra;
}

Angle getPermissibleDirectionRange(Angle dir, Angle min, Angle max)
{
    return fabs(dir) / Angles::AnglePi2 * (max - min) + min;
}

}
